/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */
package com.jcg.kafkaconsumerexample;

import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;

import java.time.Duration;
import java.util.Arrays;
import java.util.Properties;
import java.util.concurrent.ExecutionException;
import org.apache.kafka.clients.admin.AdminClient;
import org.apache.kafka.clients.admin.AdminClientConfig;
import org.apache.kafka.clients.admin.NewTopic;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;


/**
 *
 * @author omozegieaziegbe
 */
public class KafkaConsumerExample {

    private static final String BOOTSTRAP_SERVERS = "localhost:9092";

    public static void main(String[] args)  throws ExecutionException, InterruptedException {

        // Create topics
        //createTopics(Arrays.asList("transactions","financial-events"));

        // Test Kafka consumer configuration and publish messages
        testKafkaConsumer();

    }
    
    private static void testKafkaConsumer() {
        
        Properties props = new Properties();
        props.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
        props.put(ConsumerConfig.GROUP_ID_CONFIG, "test-group");
        props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
        props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());        
        
        KafkaConsumer<String, String> consumer = new KafkaConsumer<>(props);
        consumer.subscribe(Arrays.asList("transactions", "financial-events")); // Use your topic names  
        
        // Create Kafka producer configuration
        Properties producerProps = new Properties();
        producerProps.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
        producerProps.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
        producerProps.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());

        // Create Kafka producer
        KafkaProducer<String, String> producer = new KafkaProducer<>(producerProps);
        
        // Publish one message on each topic
        sendMessage(producer, "transactions", "100");
        sendMessage(producer, "financial-events", "200");
        
        // Consume messages
        try {
            while (true) {
                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(10));
                for (ConsumerRecord<String, String> record : records) {
                    String message = record.value();
                    System.out.printf("Received message: key = %s, value = %s, topic = %s%n",
                            record.key(), message, record.topic());
                }
            }
        } finally {
            consumer.close();
            producer.close();
        }
        
    }
    
//private static void createTopics(Iterable<String> topics) throws ExecutionException, InterruptedException {
//        Properties props = new Properties();
//        props.put(AdminClientConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
//
//        try (AdminClient adminClient = AdminClient.create(props)) {
//            for (String topic : topics) {
//                NewTopic newTopic = new NewTopic(topic, 1, (short) 1);
//                adminClient.createTopics(Arrays.asList(newTopic)).all().get();
//            }
//        }
//    }
    
    private static void sendMessage(KafkaProducer<String, String> producer, String topic, String message) {
        // Send the message to Kafka topic
        producer.send(new ProducerRecord<>(topic, message));
        System.out.printf("Published message '%s' to topic '%s'%n", message, topic);
    }
}
